@@ -1,5 +1,5 @@
 # __init__.py
-""" the xodr module contains the classes used to generate OpenDRIVE (xml) files. 
+""" the xodr module contains the classes used to generate OpenDRIVE (xml) files.
 
 """
 from .geometry import *
@@ -9,4 +9,6 @@ from .enumerations import *
 from .links import *
 from .generators import *
 from .signals_objects import *
-from .elevation import *
\ No newline at end of file
+from .elevation import *
+from .junction_creator import *
+from .utils import *
