@@ -16,6 +16,11 @@ from scenariogeneration import prettyprint
 import xml.etree.ElementTree as ET
 
 
+@pytest.fixture(autouse=True)
+def reset_version():
+    OSC.enumerations.VersionBase().setVersion(minor=2)
+
+
 def test_endofroadcond():
     cond = OSC.EndOfRoadCondition(20)
     prettyprint(cond.get_element(), None)
@@ -60,10 +65,17 @@ def test_timeheadwaycondition():
     cond = OSC.TimeHeadwayCondition("Ego", 20, OSC.Rule.equalTo, True, False)
     prettyprint(cond.get_element())
     cond2 = OSC.TimeHeadwayCondition("Ego", 20, OSC.Rule.equalTo, True, False)
-    cond3 = OSC.TimeHeadwayCondition("Ego", 20, OSC.Rule.equalTo, True, True)
+    cond3 = OSC.TimeHeadwayCondition(
+        "Ego",
+        20,
+        OSC.Rule.equalTo,
+        True,
+        True,
+        routing_algorithm=OSC.RoutingAlgorithm.assignedRoute,
+    )
     assert cond == cond2
     assert cond != cond3
-
+    prettyprint(cond3.get_element())
     cond4 = OSC.TimeHeadwayCondition.parse(cond.get_element())
     assert cond == cond4
 
@@ -77,7 +89,11 @@ def test_timetocollisioncondition():
         value=20, alongroute=True, rule=OSC.Rule.equalTo, entity="Ego"
     )
     cond3 = OSC.TimeToCollisionCondition(
-        value=20, alongroute=True, rule=OSC.Rule.equalTo, position=OSC.WorldPosition()
+        value=20,
+        alongroute=True,
+        rule=OSC.Rule.equalTo,
+        position=OSC.WorldPosition(),
+        routing_algorithm=OSC.RoutingAlgorithm.assignedRoute,
     )
     prettyprint(cond3.get_element(), None)
     assert cond == cond2
@@ -98,6 +114,16 @@ def test_accelerationcondition():
     assert cond != cond3
     cond4 = OSC.AccelerationCondition.parse(cond.get_element())
     assert cond == cond4
+    cond5 = OSC.AccelerationCondition(
+        1, OSC.Rule.lessThan, OSC.DirectionalDimension.longitudinal
+    )
+    cond6 = OSC.AccelerationCondition(
+        1, OSC.Rule.lessThan, OSC.DirectionalDimension.lateral
+    )
+    prettyprint(cond5.get_element())
+    cond7 = OSC.AccelerationCondition.parse(cond5.get_element())
+    assert cond5 != cond6
+    assert cond5 == cond7
 
 
 def test_standstillcondition():
@@ -112,9 +138,9 @@ def test_standstillcondition():
 
 
 def test_speedcondition():
-    cond = OSC.SpeedCondition(1, OSC.Rule.lessThan)
+    cond = OSC.SpeedCondition(1, OSC.Rule.lessThan, OSC.DirectionalDimension.lateral)
     prettyprint(cond.get_element())
-    cond2 = OSC.SpeedCondition(1, OSC.Rule.lessThan)
+    cond2 = OSC.SpeedCondition(1, OSC.Rule.lessThan, OSC.DirectionalDimension.lateral)
     cond3 = OSC.SpeedCondition(2, OSC.Rule.lessThan)
     assert cond == cond2
     assert cond != cond3
@@ -124,9 +150,13 @@ def test_speedcondition():
 
 
 def test_relativespeedcondition():
-    cond = OSC.RelativeSpeedCondition(1, OSC.Rule.lessThan, "Ego")
+    cond = OSC.RelativeSpeedCondition(
+        1, OSC.Rule.lessThan, "Ego", OSC.DirectionalDimension.lateral
+    )
     prettyprint(cond.get_element())
-    cond2 = OSC.RelativeSpeedCondition(1, OSC.Rule.lessThan, "Ego")
+    cond2 = OSC.RelativeSpeedCondition(
+        1, OSC.Rule.lessThan, "Ego", OSC.DirectionalDimension.lateral
+    )
     cond3 = OSC.RelativeSpeedCondition(1, OSC.Rule.lessThan, "Ego2")
     assert cond == cond2
     assert cond != cond3
@@ -160,10 +190,22 @@ def test_reachpositioncondition():
 
 
 def test_distancecondition():
-    cond = OSC.DistanceCondition(1, OSC.Rule.lessThan, OSC.WorldPosition(), True, False)
+    cond = OSC.DistanceCondition(
+        1,
+        OSC.Rule.lessThan,
+        OSC.WorldPosition(),
+        True,
+        False,
+        routing_algorithm=OSC.RoutingAlgorithm.fastest,
+    )
     prettyprint(cond.get_element())
     cond2 = OSC.DistanceCondition(
-        1, OSC.Rule.lessThan, OSC.WorldPosition(), True, False
+        1,
+        OSC.Rule.lessThan,
+        OSC.WorldPosition(),
+        True,
+        False,
+        routing_algorithm=OSC.RoutingAlgorithm.fastest,
     )
     cond3 = OSC.DistanceCondition(
         1, OSC.Rule.greaterThan, OSC.WorldPosition(), True, False
@@ -177,11 +219,23 @@ def test_distancecondition():
 
 def test_relativedistancecondition():
     cond = OSC.RelativeDistanceCondition(
-        1, OSC.Rule.equalTo, OSC.RelativeDistanceType.longitudinal, "Ego", True, False
+        1,
+        OSC.Rule.equalTo,
+        OSC.RelativeDistanceType.longitudinal,
+        "Ego",
+        True,
+        False,
+        routing_algorithm=OSC.RoutingAlgorithm.fastest,
     )
     prettyprint(cond.get_element())
     cond2 = OSC.RelativeDistanceCondition(
-        1, OSC.Rule.equalTo, OSC.RelativeDistanceType.longitudinal, "Ego", True, False
+        1,
+        OSC.Rule.equalTo,
+        OSC.RelativeDistanceType.longitudinal,
+        "Ego",
+        True,
+        False,
+        routing_algorithm=OSC.RoutingAlgorithm.fastest,
     )
     cond3 = OSC.RelativeDistanceCondition(
         1, OSC.Rule.equalTo, OSC.RelativeDistanceType.longitudinal, "Ego", True, True
@@ -195,6 +249,7 @@ def test_relativedistancecondition():
 
 def test_parametercondition():
     cond = OSC.ParameterCondition("MyParam", 1, OSC.Rule.equalTo)
+    cond.setVersion(minor=1)
     prettyprint(cond.get_element())
     cond2 = OSC.ParameterCondition("MyParam", 1, OSC.Rule.equalTo)
     cond3 = OSC.ParameterCondition("MyParam", 1, OSC.Rule.lessThan)
@@ -205,6 +260,18 @@ def test_parametercondition():
     assert cond == cond4
 
 
+def test_variablecondition():
+    cond = OSC.VariableCondition("MyParam", 1, OSC.Rule.equalTo)
+    prettyprint(cond.get_element())
+    cond2 = OSC.VariableCondition("MyParam", 1, OSC.Rule.equalTo)
+    cond3 = OSC.VariableCondition("MyParam", 1, OSC.Rule.lessThan)
+    assert cond == cond2
+    assert cond != cond3
+
+    cond4 = OSC.VariableCondition.parse(cond.get_element())
+    assert cond == cond4
+
+
 def test_timeofdaycondition():
     cond = OSC.TimeOfDayCondition(OSC.Rule.equalTo, "12")
     prettyprint(cond.get_element())
@@ -277,6 +344,34 @@ def test_trafficsignalcondition():
     assert cond == cond4
 
 
+def test_relaticeclearancecondition():
+    cond = OSC.RelativeClearanceCondition(True, -1, 3)
+    cond.add_entity("ego")
+    cond.add_relative_lane_range(1, 2)
+    prettyprint(cond.get_element())
+
+    cond2 = OSC.RelativeClearanceCondition(True, -1, 3)
+    cond2.add_entity("ego")
+    cond2.add_relative_lane_range(1, 2)
+
+    cond3 = OSC.RelativeClearanceCondition(True, -1, 3)
+    cond3.add_entity("ego")
+    cond3.add_entity("target")
+    cond3.add_relative_lane_range(1, 2)
+    cond3.add_relative_lane_range(5, 6)
+    prettyprint(cond3.get_element())
+
+    assert cond == cond2
+    assert cond != cond3
+
+    cond4 = OSC.RelativeClearanceCondition.parse(cond.get_element())
+    assert cond == cond4
+
+    cond5 = OSC.RelativeClearanceCondition.parse(cond3.get_element())
+    prettyprint(cond5.get_element())
+    assert cond3 == cond5
+
+
 def test_trafficsignalconditioncontroller():
     cond = OSC.TrafficSignalControllerCondition("somereferens", "yellow")
     prettyprint(cond.get_element())
@@ -401,7 +496,6 @@ def test_conditiongroup():
 
 
 def test_trigger():
-
     condgr = OSC.ConditionGroup()
 
     trig1 = OSC.EntityTrigger(
