@@ -9,6 +9,7 @@
   Copyright (c) 2022 The scenariogeneration Authors.
 
 """
+import xml.etree.ElementTree as ET
 import pytest
 import datetime as dt
 
@@ -17,6 +18,11 @@ from scenariogeneration import prettyprint
 from scenariogeneration.xosc.utils import _TrafficSignalState, ValueConstraintGroup
 
 
+@pytest.fixture(autouse=True)
+def reset_version():
+    OSC.enumerations.VersionBase().setVersion(minor=2)
+
+
 @pytest.mark.parametrize(
     "teststring",
     [
@@ -32,14 +38,18 @@ def test_transition_dynamics(teststring):
     prettyprint(td.get_element())
 
     td2 = OSC.TransitionDynamics(OSC.DynamicsShapes.step, teststring, 1)
-    td3 = OSC.TransitionDynamics(OSC.DynamicsShapes.step, teststring, 2)
+    td3 = OSC.TransitionDynamics(
+        OSC.DynamicsShapes.step, teststring, 2, following_mode=OSC.FollowingMode.follow
+    )
     prettyprint(td2.get_element())
     prettyprint(td3.get_element())
     assert td == td2
     assert td != td3
 
     td4 = OSC.TransitionDynamics.parse(td.get_element())
-    td == td4
+    assert td == td4
+    td5 = OSC.TransitionDynamics.parse(td3.get_element())
+    assert td5 == td3
 
 
 @pytest.mark.parametrize(
@@ -131,11 +141,11 @@ def test_orientation_filled(testinp, results):
 
 
 def test_parameter():
-    param = OSC.Parameter("stuffs", OSC.ParameterType.integer, "1")
+    param = OSC.Parameter("stuffs", OSC.ParameterType.int, "1")
     prettyprint(param.get_element())
-    param2 = OSC.Parameter("stuffs", OSC.ParameterType.integer, "1")
-    param3 = OSC.Parameter("stuffs", OSC.ParameterType.integer, "2")
-    param4 = OSC.Parameter("stuffs", OSC.ParameterType.integer, "1")
+    param2 = OSC.Parameter("stuffs", OSC.ParameterType.int, "1")
+    param3 = OSC.Parameter("stuffs", OSC.ParameterType.int, "2")
+    param4 = OSC.Parameter("stuffs", OSC.ParameterType.int, "1")
     assert param == param2
     assert param != param3
     param5 = OSC.Parameter.parse(param.get_element())
@@ -157,6 +167,18 @@ def test_parameter():
     assert param4 == param6
 
 
+def test_variable():
+    param = OSC.Variable("stuffs", OSC.ParameterType.int, "1")
+    prettyprint(param.get_element())
+    param2 = OSC.Variable("stuffs", OSC.ParameterType.int, "1")
+    param3 = OSC.Variable("stuffs", OSC.ParameterType.int, "2")
+    param4 = OSC.Variable("stuffs", OSC.ParameterType.int, "1")
+    assert param == param2
+    assert param != param3
+    param5 = OSC.Variable.parse(param.get_element())
+    assert param == param5
+
+
 def test_catalogreference():
     catref = OSC.CatalogReference("VehicleCatalog", "S60")
     prettyprint(catref.get_element())
@@ -180,23 +202,39 @@ def test_catalogreference():
 
 
 def test_paramdeclaration():
-
     pardec = OSC.ParameterDeclarations()
-    pardec.add_parameter(OSC.Parameter("myparam1", OSC.ParameterType.integer, "1"))
+    pardec.add_parameter(OSC.Parameter("myparam1", OSC.ParameterType.int, "1"))
     pardec.add_parameter(OSC.Parameter("myparam1", OSC.ParameterType.double, "0.01"))
     pardec2 = OSC.ParameterDeclarations()
-    pardec2.add_parameter(OSC.Parameter("myparam1", OSC.ParameterType.integer, "1"))
+    pardec2.add_parameter(OSC.Parameter("myparam1", OSC.ParameterType.int, "1"))
     pardec2.add_parameter(OSC.Parameter("myparam1", OSC.ParameterType.double, "0.01"))
     pardec3 = OSC.ParameterDeclarations.parse(pardec.get_element())
     prettyprint(pardec.get_element())
     assert pardec == pardec2
     assert pardec == pardec3
     pardec4 = OSC.ParameterDeclarations()
-    pardec4.add_parameter(OSC.Parameter("myparam2", OSC.ParameterType.integer, "1"))
+    pardec4.add_parameter(OSC.Parameter("myparam2", OSC.ParameterType.int, "1"))
     pardec4.add_parameter(OSC.Parameter("myparam2", OSC.ParameterType.double, "0.01"))
     assert pardec4 != pardec
 
 
+def test_variabledeclaration():
+    pardec = OSC.VariableDeclarations()
+    pardec.add_variable(OSC.Variable("myparam1", OSC.ParameterType.int, "1"))
+    pardec.add_variable(OSC.Variable("myparam1", OSC.ParameterType.double, "0.01"))
+    pardec2 = OSC.VariableDeclarations()
+    pardec2.add_variable(OSC.Variable("myparam1", OSC.ParameterType.int, "1"))
+    pardec2.add_variable(OSC.Variable("myparam1", OSC.ParameterType.double, "0.01"))
+    pardec3 = OSC.VariableDeclarations.parse(pardec.get_element())
+    prettyprint(pardec.get_element())
+    assert pardec == pardec2
+    assert pardec == pardec3
+    pardec4 = OSC.VariableDeclarations()
+    pardec4.add_variable(OSC.Variable("myparam2", OSC.ParameterType.int, "1"))
+    pardec4.add_variable(OSC.Variable("myparam2", OSC.ParameterType.double, "0.01"))
+    assert pardec4 != pardec
+
+
 def test_entityref():
     entref = OSC.EntityRef("ref_str")
     entref2 = OSC.EntityRef("ref_str")
@@ -280,11 +318,15 @@ def test_controller():
     prop.add_property("mything", "2")
     prop.add_property("theotherthing", "true")
 
-    param = OSC.Parameter("stuffs", OSC.ParameterType.integer, "1")
+    param = OSC.Parameter("stuffs", OSC.ParameterType.int, "1")
     param2 = OSC.Parameter("stuffs2", OSC.ParameterType.double, "5")
-    cnt = OSC.Controller("mycontroler", prop)
+    cnt = OSC.Controller(
+        "mycontroler", prop, controller_type=OSC.ControllerType.appearance
+    )
     prettyprint(cnt.get_element())
-    cnt2 = OSC.Controller("mycontroler", prop)
+    cnt2 = OSC.Controller(
+        "mycontroler", prop, controller_type=OSC.ControllerType.appearance
+    )
     cnt3 = OSC.Controller("mycontroler3", prop)
     assert cnt == cnt2
     assert cnt != cnt3
@@ -321,16 +363,16 @@ def test_timeref():
 
 
 def test_phase():
-    p1 = OSC.Phase("myphase", 1)
+    p1 = OSC.Phase("myphase", 1, traffic_group_state="hello")
     prettyprint(p1.get_element())
     p1.add_signal_state("myid", "red")
     p1.add_signal_state("myid", "green")
     prettyprint(p1.get_element())
-    p2 = OSC.Phase("myphase", 1)
+    p2 = OSC.Phase("myphase", 1, traffic_group_state="hello")
     p2.add_signal_state("myid", "red")
     p2.add_signal_state("myid", "green")
 
-    p3 = OSC.Phase("myphase", 1)
+    p3 = OSC.Phase("myphase", 1, traffic_group_state="hello")
     p3.add_signal_state("myid", "red")
 
     assert p1 == p2
@@ -408,17 +450,25 @@ def test_trafficdefinition():
     traffic3 = OSC.TrafficDefinition("my traffic")
     traffic3.add_controller(controller, 0.5)
     traffic3.add_vehicle(OSC.VehicleCategory.car, 0.9)
+    traffic3.add_vehicle_role(OSC.Role.military, 1)
     assert traffic == traffic2
     assert traffic != traffic3
-
+    prettyprint(traffic3)
     traffic4 = OSC.TrafficDefinition.parse(traffic.get_element())
     assert traffic == traffic4
 
+    traffic5 = OSC.TrafficDefinition.parse(traffic3.get_element())
+    assert traffic3 == traffic5
+
 
 def test_weather():
-    weather = OSC.Weather(OSC.CloudState.free, 100, 0)
+    weather = OSC.Weather(
+        OSC.CloudState.free, 100, 0, dome_image="dome.jpg", dome_azimuth_offset=2
+    )
     prettyprint(weather.get_element())
-    weather2 = OSC.Weather(OSC.CloudState.free, 100, 0)
+    weather2 = OSC.Weather(
+        OSC.CloudState.free, 100, 0, dome_image="dome.jpg", dome_azimuth_offset=2
+    )
     weather3 = OSC.Weather(OSC.CloudState.free, 100, 1)
     assert weather == weather2
     assert weather != weather3
@@ -444,9 +494,9 @@ def test_roadcondition():
     prop.add_property("mything", "2")
     prop.add_property("theotherthing", "true")
     prop.add_file("propfile.xml")
-    rc = OSC.RoadCondition(1, prop)
+    rc = OSC.RoadCondition(1, prop, OSC.Wetness.highFlooded)
     prettyprint(rc.get_element())
-    rc2 = OSC.RoadCondition(1, prop)
+    rc2 = OSC.RoadCondition(1, prop, OSC.Wetness.highFlooded)
     rc3 = OSC.RoadCondition(2)
     assert rc == rc2
     assert rc != rc3
@@ -479,7 +529,7 @@ def test_environment():
 def test_oscenum():
     enum1 = OSC.enumerations._OscEnum("classname", "testname")
     assert enum1.get_name() == "testname"
-    enum2 = OSC.enumerations._OscEnum("classname", "testname", min_minor_version=2)
+    enum2 = OSC.enumerations._OscEnum("classname", "testname", min_minor_version=3)
     with pytest.raises(OSC.OpenSCENARIOVersionError):
         enum2.get_name()
     enum3 = OSC.enumerations._OscEnum("classname", "testname", max_minor_version=0)
@@ -562,14 +612,15 @@ def test_fog():
 
 
 def test_dynamicsConstraints():
-    dc = OSC.DynamicsConstraints(2, 2, 3)
-    dc2 = OSC.DynamicsConstraints(2, 2, 3)
+    dc = OSC.DynamicsConstraints(2, 2, 3, 1, 2)
+    dc2 = OSC.DynamicsConstraints(2, 2, 3, 1, 2)
     dc3 = OSC.DynamicsConstraints(3, 2, 2)
     prettyprint(dc.get_element())
     assert dc == dc2
     assert dc != dc3
 
     dc4 = OSC.DynamicsConstraints.parse(dc.get_element())
+    prettyprint(dc4.get_element())
     assert dc == dc4
 
 
@@ -696,3 +747,213 @@ def test_convert_bool():
 
     with pytest.raises(ValueError):
         OSC.convert_bool("asdf")
+
+
+def test_colorrgb():
+    c = OSC.ColorRGB(1, 2, 3)
+    c2 = OSC.ColorRGB(1, 2, 3)
+    c3 = OSC.ColorRGB(1, 2, 4)
+    prettyprint(c)
+    assert c == c2
+    assert c != c3
+    c4 = OSC.ColorRGB.parse(c.get_element())
+    assert c == c4
+
+
+def test_colorcmyk():
+    c = OSC.ColorCMYK(1, 2, 3, 1)
+    c2 = OSC.ColorCMYK(1, 2, 3, 1)
+    c3 = OSC.ColorCMYK(1, 2, 4, 2)
+    prettyprint(c)
+    assert c == c2
+    assert c != c3
+    c4 = OSC.ColorCMYK.parse(c.get_element())
+    assert c == c4
+
+
+def test_color():
+    c = OSC.Color(OSC.ColorType.black, OSC.ColorCMYK(1, 2, 3, 1))
+
+    c2 = c = OSC.Color(OSC.ColorType.black, OSC.ColorCMYK(1, 2, 3, 1))
+    c3 = OSC.Color(OSC.ColorType.black, OSC.ColorRGB(1, 2, 3))
+    prettyprint(c)
+    assert c == c2
+    assert c != c3
+    c4 = OSC.Color.parse(c.get_element())
+    assert c == c4
+
+
+def test_userdefinedlight():
+    udl = OSC.UserDefinedLight("superlight")
+    udl2 = OSC.UserDefinedLight("superlight")
+    udl3 = OSC.UserDefinedLight("less super light")
+    prettyprint(udl)
+    assert udl == udl2
+    assert udl != udl3
+    udl4 = OSC.UserDefinedLight.parse(udl.get_element())
+    assert udl4 == udl
+
+
+def test_lightstate():
+    ls = OSC.utils._LightState(
+        OSC.LightMode.on,
+        color=OSC.Color(OSC.ColorType.black, OSC.ColorRGB(0, 0, 0)),
+        intensity=200,
+        flashing_off_duration=0.3,
+        flashing_on_duration=0.2,
+    )
+    ls2 = OSC.utils._LightState(
+        OSC.LightMode.on,
+        color=OSC.Color(OSC.ColorType.black, OSC.ColorRGB(0, 0, 0)),
+        intensity=200,
+        flashing_off_duration=0.3,
+        flashing_on_duration=0.2,
+    )
+    ls3 = OSC.utils._LightState(OSC.LightMode.flashing)
+    prettyprint(ls)
+    print(ls3)
+    prettyprint(ls3)
+    assert ls == ls2
+    assert ls != ls3
+    ls4 = OSC.utils._LightState.parse(ls.get_element())
+    assert ls == ls4
+
+
+def test_fileheader():
+    fh = OSC.FileHeader("my_scenario", "Mandolin", creation_date=dt.datetime.now())
+    prettyprint(fh.get_element())
+    fh2 = OSC.FileHeader("my_scenario", "Mandolin")
+    props = OSC.Properties()
+    props.add_property("prop1", "something")
+    fh3 = OSC.FileHeader("my_scenario", "Mandolin2", properties=props)
+    assert fh == fh2
+    assert fh != fh3
+
+    fh4 = OSC.FileHeader.parse(fh.get_element())
+    assert fh4 == fh
+
+
+def test_animationfile():
+    ani = OSC.AnimationFile("file_ref")
+    ani2 = OSC.AnimationFile("file_ref", 1.5)
+    prettyprint(ani2.get_element())
+    ani3 = OSC.AnimationFile("file_ref2", 1.5)
+    assert ani != ani2
+    assert ani2 != ani3
+
+    ani4 = OSC.AnimationFile.parse(ani2.get_element())
+    prettyprint(ani4.get_element())
+    assert ani4 == ani2
+
+
+def test_directionoftraveldistribution():
+    dotd = OSC.DirectionOfTravelDistribution(1, 2)
+    dotd2 = OSC.DirectionOfTravelDistribution(1, 2)
+    prettyprint(dotd.get_element())
+    dotd3 = OSC.DirectionOfTravelDistribution(1, 1)
+    assert dotd == dotd2
+    assert dotd != dotd3
+
+    dotd4 = OSC.DirectionOfTravelDistribution.parse(dotd.get_element())
+    prettyprint(dotd4.get_element())
+    assert dotd4 == dotd
+
+
+def test_userdefinedanimation():
+    ani = OSC.UserDefinedAnimation("animation_type")
+    ani2 = OSC.UserDefinedAnimation("animation_type2")
+    ani3 = OSC.UserDefinedAnimation("animation_type")
+    prettyprint(ani.get_element())
+    assert ani != ani2
+    assert ani == ani3
+    ani4 = OSC.UserDefinedAnimation.parse(ani.get_element())
+    assert ani == ani4
+
+
+def test_userdefinedcomponent():
+    ani = OSC.UserDefinedComponent("component_type")
+    ani2 = OSC.UserDefinedComponent("component_type2")
+    ani3 = OSC.UserDefinedComponent("component_type")
+    prettyprint(ani.get_element())
+    assert ani != ani2
+    assert ani == ani3
+    ani4 = OSC.UserDefinedComponent.parse(ani.get_element())
+    assert ani == ani4
+
+
+def test_pedestriananimation():
+    pa = OSC.PedestrianAnimation(OSC.PedestrianMotionType.running, "animation1")
+    pa2 = OSC.PedestrianAnimation(OSC.PedestrianMotionType.running, "animation2")
+    pa3 = OSC.PedestrianAnimation(OSC.PedestrianMotionType.ducking, "animation1")
+    prettyprint(pa.get_element())
+    assert pa != pa2
+    assert pa != pa3
+    pa4 = OSC.PedestrianAnimation(OSC.PedestrianMotionType.running, "animation1")
+    pa.add_gesture(OSC.PedestrianGestureType.sandwichLeftHand)
+    pa4.add_gesture(OSC.PedestrianGestureType.sandwichRightHand)
+    assert pa != pa4
+    pa.add_gesture(OSC.PedestrianGestureType.sandwichRightHand)
+    prettyprint(pa.get_element())
+    pa5 = OSC.PedestrianAnimation.parse(pa.get_element())
+    assert pa5 == pa
+
+
+def test_vehiclecomponent():
+    vc = OSC.utils._VehicleComponent(OSC.VehicleComponentType.doorFrontLeft)
+    vc2 = OSC.utils._VehicleComponent(OSC.VehicleComponentType.doorRearLeft)
+    prettyprint(vc.get_element())
+    assert vc != vc2
+    vc3 = OSC.utils._VehicleComponent(OSC.VehicleComponentType.doorFrontLeft)
+    assert vc == vc3
+    vc4 = OSC.utils._VehicleComponent.parse(vc.get_element())
+    prettyprint(vc4.get_element())
+    assert vc4 == vc
+
+
+def test_componentanimation():
+    vc = OSC.utils._VehicleComponent(OSC.VehicleComponentType.doorFrontLeft)
+    vc2 = OSC.utils._VehicleComponent(OSC.VehicleComponentType.doorRearRight)
+    udc = OSC.UserDefinedComponent("my_component")
+    udc2 = OSC.UserDefinedComponent("my_component2")
+    udc3 = OSC.UserDefinedComponent("doorFrontLeft")
+    ca = OSC.utils._ComponentAnimation(vc)
+    prettyprint(ca.get_element())
+    ca2 = OSC.utils._ComponentAnimation(vc2)
+    assert ca != ca2
+    ca3 = OSC.utils._ComponentAnimation(udc)
+    prettyprint(ca3.get_element())
+    ca4 = OSC.utils._ComponentAnimation(udc3)
+    prettyprint(ca4.get_element())
+    assert ca != ca3
+    assert ca != ca4
+    ca5 = OSC.utils._ComponentAnimation(udc2)
+    assert ca3 != ca5
+
+    ca6 = OSC.utils._ComponentAnimation.parse(ca.get_element())
+    prettyprint(ca6.get_element())
+    assert ca6 == ca
+    ca7 = OSC.utils._ComponentAnimation.parse(ca2.get_element())
+    assert ca7 == ca2
+    ca8 = OSC.utils._ComponentAnimation.parse(ca4.get_element())
+    assert ca8 == ca4
+    assert ca8 != ca
+
+
+@pytest.mark.parametrize(
+    "input",
+    [
+        OSC.utils._ComponentAnimation(
+            OSC.utils._VehicleComponent(OSC.VehicleComponentType.doorFrontLeft)
+        ),
+        OSC.PedestrianAnimation(OSC.PedestrianMotionType.bendingDown),
+        OSC.AnimationFile("my_animation"),
+        OSC.UserDefinedAnimation("Myparam"),
+    ],
+)
+def test_global_action_factory(input):
+    test_element = ET.Element("AnimationType")
+    test_element.append(input.get_element())
+    factoryoutput = OSC.utils._AnimationTypeFactory.parse_animationtype(test_element)
+    prettyprint(input, None)
+    prettyprint(factoryoutput, None)
+    assert input == factoryoutput
